/*
  Babix - An educational little preemptive multitask kernel for Arduino Due.
  Fran�ois Pessaux 04/2015.
  Maxime Ayrault 06/2015.
  This code can be freely distributed.
*/

#include <Arduino.h>
#include "semaphore.h"
#include "user_tasks2.h"
#include "process.h"

/** Global variable to increment concurrently by several processes. */
volatile uint32_t glob_shared_cpt2 = 0 ;
/** The semaphore to protect the shared variable.
    If initialized with 1, it acts as a mutex.
    It initialized with 2, then the processes smash the variable concurrently
    leading to an inconsistent value. */
volatile semaphore_t sem = 1 ;


/** Simply increment a share global variable, simulating a long processing
   between the readign of the variable and its effective increment.
   The access to the variable is protected by a critical section.
   Basically, this function does:
     read glob_shared_cpt,
     wait,
     glob_shared_cpt = glob_shared_cpt + 1,
     print glob_shared_cpt

   Arg:
    - t : time in ms to wait between reading and writing the shared global
         variable 'glob_shared_cpt'. */
void concurrent_increment2 (int t)
{
  int tmp ;

  /* Claim the lock to prevent several processes from modifying the kernel
     tables concurrently if they attempt to create processes "at the same
     time". */
  sem_acquire (&sem) ;
  tmp = glob_shared_cpt2 ;
  /* Wait t ms to simulate a looonnnng processing ^^ */
  delay (t) ;
  glob_shared_cpt2 = tmp + 1 ;
  Serial.println (glob_shared_cpt2) ;
  /* Release the mutex to leave the critical section. */
  sem_release (&sem) ;
  return ;
}



/** Some examples of user processes. */
void process4 ()
{
  int i = 0 ;
  for (i = 0; i < 20; i ++) {
    Serial.print ("I'm 0 ") ;
    concurrent_increment2 (15) ;
    delay (70) ;
  }
  end_process () ;
}

void process5 ()
{
  int i = 0 ;
  for (i = 0; i < 30; i ++) {
    Serial.print ("I'm 1 ") ;
    concurrent_increment2 (99) ;
    delay (470) ;
  }
  end_process () ;
}

void process6 ()
{
  int i = 0 ;
  for (i = 0; i < 8; i ++) {
    Serial.print ("I'm 2 ") ;
    concurrent_increment2 (24) ;
    delay (3700) ;
  }
  end_process () ;
}

void process7 ()
{
  int i = 0 ;
  for (i = 0; i < 9; i ++) {
    digitalWrite (13, HIGH) ;
    delay (1000) ;
    digitalWrite (13, LOW) ;
    delay (1000) ;
  }
  end_process () ;
}
